﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Sailboat.Entities;
using Sailboat.MovementStrategy;

namespace Sailboat.DrawingObjects
{
    public class DrawingBoat
    {
        public EntityBoat? EntityBoat { get; protected set; }
        private int _pictureWidth;
        private int _pictureHeight;
        protected int _startPosX;
        protected int _startPosY;
        private readonly int _boatWidth = 160;
        private readonly int _boatHeight = 160;
        public int GetPosX => _startPosX;
        public int GetPosY => _startPosY;
        public int GetWidth => _boatWidth;
        public int GetHeight => _boatHeight;
        public IMoveableObject GetMoveableObject => new DrawingObjectBoat(this);
        public DrawingBoat(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _boatWidth || height < _boatHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }

        protected DrawingBoat(int speed, double weight, Color bodyColor, int width, int height, int boatWidth, int boatHeight)
        {
            if (width < _boatWidth || height < _boatHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            _boatWidth = boatWidth;
            _boatHeight = boatHeight;
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }
        public void SetPosition(int x, int y)
        {
            if (x < 0 || x + _boatWidth > _pictureWidth)
            {
                x = 0;
            }
            if (y < 0 || y + _boatHeight > _pictureHeight)
            {
                y = 0;
            }
            _startPosX = x;
            _startPosY = y;
        }
        public bool CanMove(DirectionType direction)
        {
            if (EntityBoat == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityBoat.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityBoat.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + EntityBoat.Step < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + EntityBoat.Step < _pictureHeight,
                _ => false
            };
        }
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityBoat == null)
            {
                return;
            }
            switch (direction)
            {
                case DirectionType.Left:
                    if (_startPosX - EntityBoat.Step > 0)
                    {
                        _startPosX -= (int)EntityBoat.Step;
                    }
                    break;
                case DirectionType.Up:
                    if (_startPosY - EntityBoat.Step > 0)
                    {
                        _startPosY -= (int)EntityBoat.Step;
                    }
                    break;
                case DirectionType.Right:
                    if (_startPosX + EntityBoat.Step + _boatWidth < _pictureWidth)
                    {
                        _startPosX += (int)EntityBoat.Step;
                    }
                    break;
                case DirectionType.Down:
                    if (_startPosY + EntityBoat.Step + _boatHeight < _pictureHeight)
                    {
                        _startPosY += (int)EntityBoat.Step;
                    }
                    break;
            }
        }
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityBoat == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new
            SolidBrush(EntityBoat.BodyColor);

            //основной корпус парусника 
            Brush Brush = new
            SolidBrush(EntityBoat.BodyColor);

            Point[] hull = new Point[]
            {
                new Point(_startPosX + 10, _startPosY + 90),
                new Point(_startPosX + 110, _startPosY + 90),
                new Point(_startPosX + 140, _startPosY + 120),
                new Point(_startPosX + 110, _startPosY + 150),
                new Point(_startPosX + 10, _startPosY + 150)
            };
            g.FillPolygon(Brush, hull);
            g.DrawPolygon(pen, hull);

            Brush addBrush = new
            SolidBrush(Color.Green);

            g.FillEllipse(addBrush, _startPosX + 20, _startPosY + 100, 90, 40);
            g.DrawEllipse(pen, _startPosX + 20, _startPosY + 100, 90, 40);

            }
        }
    }

